/// <reference types="node" />
/**
 * @module node-zk-treecache
 */
/**  */
import { CuratorFramework } from "./CuratorFramework";
import { TreeCacheSelector } from "./TreeCacheSelector";
export interface ZooKeeperStat {
    czxid: Buffer;
    mzxid: Buffer;
    ctime: Buffer;
    mtime: Buffer;
    version: number;
    cversion: number;
    aversion: number;
    ephemeralOwner: Buffer;
    dataLength: number;
    numChildren: number;
    pzxid: Buffer;
}
export interface ChildData {
    readonly path: string;
    readonly stat: ZooKeeperStat | null;
    readonly data: Buffer | undefined;
}
export declare function compareChildData(cd1: ChildData, cd2: ChildData): boolean;
export interface TreeCache {
    /**
     * Start the cache. The cache is not started automatically. You must call this method.
     *
     * @return this once initialized
     * @throws Exception errors
     */
    start(): Promise<TreeCache>;
    /**
     * Close/end the cache.
     */
    close(): void;
    /**
     * Return the current set of children at the given path, mapped by child name. There are no
     * guarantees of accuracy; this is merely the most recent view of the data.  If there is no
     * node at this path, {@code null} is returned.
     *
     * @param fullPath full path to the node to check
     * @return a possibly-empty list of children if the node is alive, or null
     */
    getCurrentChildren(fullPath: string): Map<string, ChildData> | null;
    /**
     * Return the current data for the given path. There are no guarantees of accuracy. This is
     * merely the most recent view of the data. If there is no node at the given path,
     * {@code null} is returned.
     *
     * @param fullPath full path to the node to check
     * @return data if the node is alive, or null
     */
    getCurrentData(fullPath: string): ChildData | null;
    /**
     * Add the given listener.
     *
     * @param listener listener to add
     * @return this
     */
    addListener(listener: TreeCacheListener): TreeCache;
    /**
     * Remove the given listener.
     *
     * @param listener listener to remove
     * @return this
     */
    removeListener(listener: TreeCacheListener): TreeCache;
    addErrorListener(listener: UnhandledErrorListener): TreeCache;
    removeErrorListener(listener: UnhandledErrorListener): TreeCache;
}
export interface Logger {
    id: number;
    debug(msg: string, ...args: any[]): void;
    info(msg: string, ...args: any[]): void;
    warn(msg: string, ...args: any[]): void;
    error(msg: string, ...args: any[]): void;
}
export interface TreeCacheView {
    maxDepth: number;
    selector: TreeCacheSelector;
    cacheData: boolean;
    isInitialized: boolean;
    client: CuratorFramework;
    outstandingOps: number;
    treeState(): TreeState;
    publishEventType(type: TreeCacheEventType): void;
    publishEventTypeWithData(type: TreeCacheEventType, childData: ChildData | null): void;
    handleException(e: Error): void;
}
export declare class TreeCacheEvent {
    private _type;
    private _data;
    private _date;
    constructor(_type: TreeCacheEventType, _data: ChildData | null, _date: Date);
    readonly type: TreeCacheEventType;
    readonly data: ChildData | null;
    readonly path: string | null;
    readonly date: Date;
    toString(): string;
}
export declare enum TreeCacheEventType {
    /**
     * A node was added.
     */
    NODE_ADDED = 0,
    /**
       * A node's data was changed
       */
    NODE_UPDATED = 1,
    /**
       * A node was removed from the tree
       */
    NODE_REMOVED = 2,
    /**
       * Called when the connection has changed to {@link ConnectionState#SUSPENDED}
       * <p>
       * This is exposed so that users of the class can be notified of issues that *might* affect normal operation.
       * The TreeCache is written such that listeners are not expected to do anything special on this
       * event, except for those people who want to cause some application-specific logic to fire when this occurs.
       * While the connection is down, the TreeCache will continue to have its state from before it lost
       * the connection and after the connection is restored, the TreeCache will emit normal child events
       * for all of the adds, deletes and updates that happened during the time that it was disconnected.
       * </p>
       */
    CONNECTION_SUSPENDED = 3,
    /**
       * Called when the connection has changed to {@link ConnectionState#RECONNECTED}
       * <p>
       * This is exposed so that users of the class can be notified of issues that *might* affect normal operation.
       * The TreeCache is written such that listeners are not expected to do anything special on this
       * event, except for those people who want to cause some application-specific logic to fire when this occurs.
       * While the connection is down, the TreeCache will continue to have its state from before it lost
       * the connection and after the connection is restored, the TreeCache will emit normal child events
       * for all of the adds, deletes and updates that happened during the time that it was disconnected.
       * </p><p>
       * After reconnection, the cache will resynchronize its internal state with the server, then fire a
       * {@link #INITIALIZED} event.
       * </p>
       */
    CONNECTION_RECONNECTED = 4,
    /**
       * Called when the connection has changed to {@link ConnectionState#LOST}
       * <p>
       * This is exposed so that users of the class can be notified of issues that *might* affect normal operation.
       * The TreeCache is written such that listeners are not expected to do anything special on this
       * event, except for those people who want to cause some application-specific logic to fire when this occurs.
       * While the connection is down, the TreeCache will continue to have its state from before it lost
       * the connection and after the connection is restored, the TreeCache will emit normal child events
       * for all of the adds, deletes and updates that happened during the time that it was disconnected.
       * </p>
       */
    CONNECTION_LOST = 5,
    /**
       * Posted after the initial cache has been fully populated.
       * <p>
       * On startup, the cache synchronizes its internal
       * state with the server, publishing a series of {@link #NODE_ADDED} events as new nodes are discovered.  Once
       * the cachehas been fully synchronized, this {@link #INITIALIZED} this event is published.  All events
       * published after this event represent actual server-side mutations.
       * </p><p>
       * On reconnection, the cache will resynchronize its internal state with the server, and fire this event again
       * once its internal state is completely refreshed.
       * </p><p>
       * Note: because the initial population is inherently asynchronous, so it's possible to observe server-side changes
       * (such as a {@link #NODE_UPDATED}) prior to this event being published.
       * </p>
       */
    INITIALIZED = 6,
}
export declare type TreeCacheListener = (client: CuratorFramework, event: TreeCacheEvent) => void;
export declare type UnhandledErrorListener = (message: string, error: Error) => void;
/**
 * @private
 */
export declare enum TreeState {
    LATENT = 0,
    STARTED = 1,
    CLOSED = 2,
}
/**
 * @private
 */
export declare class TreeCacheImpl implements TreeCache, TreeCacheView {
    client: CuratorFramework;
    private logger;
    cacheData: boolean;
    maxDepth: number;
    private createParentNodes;
    selector: TreeCacheSelector;
    outstandingOps: number;
    isInitialized: boolean;
    private root;
    private _treeState;
    private listeners;
    private errorListeners;
    constructor(client: CuratorFramework, logger: Logger, path: string, cacheData: boolean, maxDepth: number, createParentNodes: boolean, selector: TreeCacheSelector, outstandingOps?: number);
    treeState(): TreeState;
    start(): Promise<TreeCache>;
    close(): void;
    getCurrentChildren(fullPath: string): Map<string, ChildData> | null;
    getCurrentData(fullPath: string): ChildData | null;
    addListener(listener: TreeCacheListener): TreeCache;
    removeListener(listener: TreeCacheListener): TreeCache;
    addErrorListener(listener: UnhandledErrorListener): TreeCache;
    removeErrorListener(listener: UnhandledErrorListener): TreeCache;
    handleException(e: Error): void;
    publishEventType(type: TreeCacheEventType): void;
    publishEventTypeWithData(type: TreeCacheEventType, data: ChildData | null): void;
    private connectionStateListener;
    private find(findPath);
    private handleStateChange(newState);
    private publishEvent(ev);
}
